==> bigloo3.4a-3.tar.gz: bigloo3.4a-3/api/crypto/src/Llib/dsa.scm <==
;*=====================================================================*/
;*    Author      :  Florian Loitsch                                   */
;*    Copyright   :  2009 Florian Loitsch                              */
;*    -------------------------------------------------------------    */
;*    Message encryption and decryption based on the DSA asymmetric    */
;*    cipher.                                                          */
;*=====================================================================*/

(module __crypto-dsa
   (import __crypto-util)
   (export (class Dsa-Key
	      p::bignum q::bignum g::bignum y::bignum)
	   (final-class Complete-Dsa-Key::Dsa-Key
	    x::bignum)) ;; the private key
   
   (export (extract-public-dsa-key::Dsa-Key key::Complete-Dsa-Key)
	   (dsa-sign   key::Complete-Dsa-Key m::bignum)
	   (dsa-verify key::Dsa-Key m::bignum r::bignum s::bignum)))

(define (extract-public-dsa-key key)
   (with-access::Dsa-Key key (p q g y)
      (make-Dsa-Key p q g y)))

(define (dsa-sign key m::bignum)
   (with-access::Complete-Dsa-Key key (p q g y x)
      (let* ((k (make-random-bignum (-fx (bignum-bit-length q) 1)))
	     (r (modulobx (expt-modbx g k p) q))
	     (k-1 (mod-inverse k q))
	     (s (modulobx (*bx k-1 (+bx m (*bx x r))) q)))
	 (if (or (=bx r #z0)
		 (=bx s #z0))
	     (dsa-sign key m)
	     (values r s)))))

(define (dsa-verify key m::bignum r::bignum s::bignum)
   (with-access::Dsa-Key key (p q g y)
      (and (>bx r #z0) (<bx r q)
	   (>bx s #z0) (<bx s q)
	   (let* ((w (mod-inverse s q))
		  (u1 (modulobx (*bx m w) q))
		  (u2 (modulobx (*bx r w) q))
		  (v (modulobx (modulobx (*bx (expt-modbx g u1 p)
					      (expt-modbx y u2 p))
					 p)
			       q)))
	      (=bx v r)))))

==> bigloo3.7a-2.tar.gz: bigloo3.7a-2/api/crypto/src/Llib/dsa.scm <==
;*=====================================================================*/
;*    Author      :  Florian Loitsch                                   */
;*    Copyright   :  2009-10 Florian Loitsch                           */
;*    -------------------------------------------------------------    */
;*    Message encryption and decryption based on the DSA asymmetric    */
;*    cipher.                                                          */
;*=====================================================================*/

(module __crypto-dsa
   (import __crypto-util)
   (export (class Dsa-Key
	      p::bignum q::bignum g::bignum y::bignum)
	   (class Complete-Dsa-Key::Dsa-Key
	    x::bignum)) ;; the private key
   
   (export (extract-public-dsa-key::Dsa-Key key::Complete-Dsa-Key)
	   (dsa-sign   key::Complete-Dsa-Key m::bignum)
	   (dsa-verify key::Dsa-Key m::bignum r::bignum s::bignum)))

(define (extract-public-dsa-key key)
   (with-access::Dsa-Key key (p q g y)
      (make-Dsa-Key p q g y)))

(define (dsa-sign key m::bignum)
   (with-access::Complete-Dsa-Key key (p q g y x)
      (let* ((k (make-random-bignum (-fx (bignum-bit-length q) 1)))
	     (r (modulobx (expt-modbx g k p) q))
	     (k-1 (mod-inverse k q))
	     (s (modulobx (*bx k-1 (+bx m (*bx x r))) q)))
	 (if (or (=bx r #z0)
		 (=bx s #z0))
	     (dsa-sign key m)
	     (values r s)))))

(define (dsa-verify key m::bignum r::bignum s::bignum)
   (with-access::Dsa-Key key (p q g y)
      (and (>bx r #z0) (<bx r q)
	   (>bx s #z0) (<bx s q)
	   (let* ((w (mod-inverse s q))
		  (u1 (modulobx (*bx m w) q))
		  (u2 (modulobx (*bx r w) q))
		  (v (modulobx (modulobx (*bx (expt-modbx g u1 p)
					      (expt-modbx y u2 p))
					 p)
			       q)))
	      (=bx v r)))))
